
/**
 * Interpolation de Lagrange
 * @author Tachouct Mustapha
 */
public class Lagrange implements  InterpolationImpl {

    /**
     * les Xi
     */
    private double X[];

    /**
     * les Fi
     */
    private double F[];

    /**
     * table des diviseurs
     */
    private double DIV[];

    /**
     * nombre de points
     */
    private int N;

    /**
     * nombre maximum de points
     */
    private int MAX_N;
    
    /***************************************************************/

    /**
     * constructeur
     */
    Lagrange(double x[] , double f[])
    {X=x;F=f;N=x.length;MAX_N=2*N+256;setDIV();}

    
    /**
     * constructeur
     */
    Lagrange(double x[],double f[],int n)
    {X=x;F=f;N=n;MAX_N=2*N+256;setDIV();}


    /**
     * constructeur
     */
    Lagrange(String fileName)
    {double [][] pts = Backup.loadPoints(fileName);
    X=pts[0];F=pts[1];N=X.length;}


    /*****************************************************************/

    /**
     * calcul les Lk(x)
     */
    private double L(int k,double x)
    { 
	double num=1;
	
	for(int i = 0 ; i < N ; i++ )
	    {
		if ( i!=k)
		    
		    num*=(x-X[i]);
			
		
	    }
	
	return num/DIV[k];
    }
    


    /**
     * initialise la table des divisions
     */
    private void  setDIV(){
	double div=1;

	DIV =new double[MAX_N];
	
	
	for (int k = 0 ; k < N ; k++){
	    div=1;
	    
	    for(int i = 0 ; i < N ; i++ )
		{
		    if ( i!=k)
			div*=(X[k]-X[i]);
		    
		}


	    DIV[k]=div;
	}
	
    }


    /**
     * retourne un Polynome Lk(x)
     */ 
    private  Polynome L( int k){
	Polynome  num , mult , res;
	double div=1;
	double [] a_num=new double[1];
	double [] p_num=new double[1];
	double [] a_mult=new double[2];
	double [] p_mult=new double[2];
	
	/* num = 1 */
	a_num[0]=1;
	p_num[0]=0;
	num = new Polynome(a_num,p_num);
	
	/* mult = (x-Xi)*/
	a_mult[0]=0;
	p_mult[0]=0;
	a_mult[1]=1;
	p_mult[1]=1;
	mult=new Polynome(a_mult,p_mult);
	

	for(int i = 0 ; i < N ; i++ )
	    {
		if ( i!=k)
		    {
			/*  num*=(x-X[i]); */
			a_mult[0]=-X[i];
			num = num.mult( num , mult );
	
		    }
	    }
	
	/* num = num / div */ 
	num.div( DIV[k]);	
	
	return num;



    }
    
    /**
     * calcul P(x)
     */
    public double P(double x)
    {
	double res = 0;
	
	for ( int  i = 0 ; i < N ; i++ )
	    {
		res+=( F[i]*L(i,x) );
	    }
	
	
	return res;
    }
    
    /****************************************************/
    


    /**
     * ajoute un point
     */
    public void addPoint(double x ,double fx){
	
	/* si x existe deja : on arrete  */
	for (int i =0 ; i < N ; i++)
	    {
		if ( X[i]==x)
		    return;
	    }
	
	/* si le tableau n'est pas assez grand  */
	if ( X.length>=N || F.length>=N){
	    double newX []= new double [MAX_N];
	    double newF []= new double [MAX_N];
	    
	    System.arraycopy(X,0,newX,0,N);
	    System.arraycopy(F,0,newF,0,N);
	    
	    X=newX;
	    F=newF;
	}
	
	/* on ajoute le point */
	X[N]=x;	F[N]=fx;DIV[N]=1;
	
	
	/* on met a jour la table des divisions */
	for (int k = 0 ; k < N ; k++){
	 
	    DIV[k]*=(X[k]-x);
	    DIV[N]*=(X[k]-x);
	}
	
	
	
	N++;
    }
    
    
    /*************************************************/
    
    
    /**
     * retourne un vrai Polynome de degree N
     */
    public Polynome getPolynome(){
	Polynome res,Li;
	
	
	/* res = 0 */
	res=new Polynome(new double[0],new double[0]);
    
	
	for ( int  i = 0 ; i < N ; i++ )
	    {
		/*res+=( F[i]*L(i,x) );*/

		Li = L(i);

		Li.mult(F[i]);

		res = res.add ( res ,Li );

	    }
	

	return res;
	
    }

    /**
     * retourne l'erreur
     */
    public double getError(double x,double M){
	double res=1;
	int fact=1;
	
	for(int i = 0 ; i  <N ; i++ ){
	    res*=((x-X[i])*M);
	    fact*=(2+i);
	}

	return res/fact;
    }



    public String toString(){
	String res="";
	return "Interpolation de Lagrange";
    }
}

