

/**
 * Interpoltation de Newton
 * @author Tachouct Mustapha
 */
public class Newton implements InterpolationImpl {

    /**
     * les Xi
     */
    private double X[];

    /**
     * les Fi
     */
    private double F[];

    /**
     * les Ai
     */
    private double A[];

    /**
     * le tableau des differences divises
     */
    private double DD[][];

    /**
     * le nombre de points
     */
    private int N;

    /**
     * le nombre de points maximum
     */
    private int MAX_N;
    

    /**
     * le temps de calcul
     */
    private long time=System.currentTimeMillis();



    /**
     * constructeur 
     */
    Newton(double x[] , double f[])
    {X=x;F=f;N=x.length;MAX_N=N+256;setDD();setA();}

    /**
     * contructeur 
     */
    Newton(double x[],double f[],int n)
    {X=x;F=f;N=n;MAX_N=2*N+256;setDD();setA();}


    /**
     * intialise les Ai
     */
    private void setA()
    {
	A = new double [MAX_N];
	
	for ( int i = 0 ; i < N ; i++ ) 
	    A[i] = DD[i][0];
	
    }
    
    
    /**
     * initialise le tableau des differences divises
     */
    private void setDD()
    { 
	
	/* a optimizer ( en recussif )*/
	double res ;

	DD = new double [MAX_N][];
	for ( int n = 0 ; n < MAX_N ;n++)
	    	DD[n] = new double [MAX_N-n];

	/* ordre 0 */
	/*DD[0]= new double [MAX_N]; */
	for ( int i = 0 ; i < N ; i++)
	    DD[0][i] = F[i];

	/* ordre 1 a N */
	for ( int n = 1 ; n <N ; n++ )
	    {
		/*DD[n] = new double [MAX_N-n];*/

		for ( int i = 0 ; i < N-n ; i++)
		    DD[n][i] = (DD[n-1][i+1]-DD[n-1][i])/ (X[i+n]-X[i]);
	    }
		
	
	

    }    
    

    /**
     * affiche des information sur le polynome 
     */
    public void print(String info){

	if ( "A".equals(info))
	    for ( int i=0 ; i < N ; i++) System.out.println("A "+i+ " = "+ A[i]);
	if ("DD".equals(info)){
	    for (int i = 0 ; i <N ; i++)
		System.out.print("\t\tDD"+i);
	    System.out.println();


	    for ( int i = 0 ; i <N ; i ++){
		System.out.print("F"+i+"\t\t");

		for ( int j = 0 ; j<N-i ; j++){
		    
		    System.out.print(DD[j][i]+"\t\t");

		}
		System.out.println();
	    }
	    

	}

	


    }

    /**************************************************************/
    public double P(double x)
    {
	double res  ;
	
	/* utilisation du schema d'Horner */

	res = A[N-1];
	
	for ( int i=N-2  ;  i>=0 ; i-- )
	    {
		res *= ( x-X[i] );
		res += ( A[i] );
	    }
	
	return res ; 
    }

    /***************************************************/

    /**
     * ajoute un point sans tout recalculer
     */
    public void addPoint(double x ,double fx){
	
	/* si x existe deja : on arrete  */
	for (int i =0 ; i < N ; i++)
	    {
		if ( X[i]==x)
		    return;
	    }
	
	/* si le tableau n'est pas assez grand  */
	if ( X.length>=N || F.length>=N){
	    double newX []= new double [MAX_N];
	    double newF []= new double [MAX_N];
	    
	    System.arraycopy(X,0,newX,0,N);
	    System.arraycopy(F,0,newF,0,N);
	    
	    X=newX;
	    F=newF;

	}

	/* on ajoute le point */
	X[N]=x;	F[N]=fx;DD[0][N]=fx;


	/* on met a jour la table des differences divises */
	for ( int n = 1 ; n <=N ; n++ ){
	    
	    int i=N-n;
	    
	    DD[n][N-n] = (DD[n-1][N-n+1]-DD[n-1][N-n])/ (X[N]-X[N-n]);
	}
	

	A[N]=DD[N][0];

	N++;
	
    }

   
    /***************************************************/


    /**
     * retourne un vrai polynome de degree N
     */
    public Polynome getPolynome(){
	Polynome res,mult,add ;

	double []a_res =new double [1];
	double []p_res =new double [1];
	double []a_mult =new double [2];
	double []p_mult =new double [2];
	double []a_add =new double [1];
	double []p_add =new double [1];
	
	
	
	/* res = A[N-1] */
	a_res[0] = A[N-1];
	p_res[0] = 0;
	res = new Polynome(a_res,p_res);

	/* polynome de multiplication : (x-X[i]) */
	a_mult[1]=1;/* x */
	p_mult[1]=1;
	a_mult[0]=0;/* Xi */
	p_mult[0]=0;
	mult = new Polynome(a_mult,p_mult);
	
	/* polynome d'addition : A[i]*/
	a_add[0]=p_add[0]=0;
	add =new Polynome(a_add,p_add);
	
	/* calcul suivant le schema d'Horner */
	for ( int i=N-2  ;  i>=0 ; i-- )
	    {
		/*res *= ( x-X[i] );*/
		a_mult[0]= -X[i];    
		res=res.mult( res , mult );
		
		
		/*res += ( A[i] );*/
		a_add[0]=A[i];
		res=res.add( res , add  );   
		
		
	    }
	
	return res ; 

    }
    
    /**
     * retourne l'erreur
     */
    public double getError(double x,double M){
	double res=1;
	int fact=1;

	for(int i = 0 ; i  <N ; i++ ){
	    res*=((x-X[i])*M);
	    fact*=(2+i);
	}

	return res/fact;
    }


    public String toString(){
	String res="";
	final int SIZE = 32;

	for (int i = 0 ; i< N ; i++)
	   res+="A"+i+  "= "+ A[i]+"\n";
	
	res+=toString(" ",SIZE);
	for (int i = 0 ; i <N ; i++)
	    res+=toString("DD"+i,SIZE);
	
	res+="\n\n";
	
	for ( int i = 0 ; i <N ; i ++){
	    
	    res+=toString("F"+i,SIZE);
	    
	    for ( int j = 0 ; j<N-i ; j++)
		res+=(toString(""+DD[j][i],SIZE));
	    
	    
	    res+="\n";
	}
	
	return res;
    }

    public void setTime(){
	time = System.currentTimeMillis()-time;
    }
   
    public String toString(String str,int n){
	String res=""+str;
	for ( int i=res.length(); i<n;i++) res+=" ";
	return res;
	
    }
}





/********************************
  x =     2 3 4 
  f(x)=   4 9 16
  Newton :
  A[0]=4.0
  A[1]=5.0
  A[2]=1.0
  4.0     9.0     16.0
  5.0     7.0
  1.0
  x0 =    2.5
  P(x0)= 6.25
**********************************/ 
    
