
import java.text.*;
import java.util.*;


/**
 * Cette classe permet de faire des calculs simple sur des polynomes
 * @author Tachouct Mustapha
 */
public class Polynome {
    


    /**
     * coefficients
     */
    private double [] a; 

    /**
     * puissances
     */
    private double [] p; 

   
    private int N;

    /***************************************************/

    
    /**
     * constructeur
     */
    public Polynome ( double [] coeff,double [] puiss)
    {a=coeff;p=puiss;N=a.length;}


    /**
     * constructeur
     */
    public Polynome ( double []coeff , double [] puiss , int n)
    {a=coeff;p=puiss;N=n;}
    

    /*****************************************************/

    
    /**
     * addition entre 2 polynomes : P + Q
     */
    public static Polynome add(Polynome P, Polynome Q){
	Polynome res;
	
	double [] coeff = new double [P.N+Q.N];
	double [] puiss   = new double [P.N+Q.N];


	/*recopie : res  = P + Q*/
	System.arraycopy( P.a , 0 , coeff , 0,  P.N);
	System.arraycopy( Q.a , 0 , coeff , P.N,Q.N);
	System.arraycopy( P.p , 0 , puiss , 0,  P.N);
	System.arraycopy( Q.p , 0 , puiss , P.N,Q.N);
	
	res= new Polynome(coeff,puiss);


	res.factorise();

	return res;

    }



    /**
     * soutraction entre 2 polynomes :  P - Q
     */
    public static Polynome sub(Polynome P, Polynome Q){
	Polynome res;
	
	double [] coeff = new double [P.N+Q.N];
	double [] puiss   = new double [P.N+Q.N];


	/*recopie : res  = P - Q*/
	System.arraycopy( P.a , 0 , coeff , 0,  P.N);
	System.arraycopy( Q.a , 0 , coeff , P.N,Q.N);
	System.arraycopy( P.p , 0 , puiss , 0,  P.N);
	System.arraycopy( Q.p , 0 , puiss , P.N,Q.N);

	for ( int i = P.N ; i< P.N+Q.N ; i++)
	    coeff[i]*= -1;

	res= new Polynome(coeff,puiss);


	res.factorise();

	return res;

    }



    /**
     * somme d'une liste de Polyome
     */
    public static Polynome sum(Polynome [] tab){
	Polynome res = new Polynome( new double[0] , new double[0]);

	for (int i = 0 ; i< tab.length ; i++){
	    res=res.add(res,tab[i]);
	   
	}
	return res;
    }



    /**
     * multiplication par une constante
     */
    public void  mult( double k){
	for ( int i = 0 ; i <N  ;i++)
	    a[i]*=k;
    }
    


    /**
     * multiplication par aX^n
     */
    public void mult ( double coeff , double puiss){
	for ( int i = 0 ; i < N ; i++){
	    a[i]*=coeff;
	    p[i]+=puiss;
	}
	    
    }



    /**
     * multiplication deux polynomes : P * Q
     */
    public static Polynome mult( Polynome P,Polynome Q){
	Polynome [] tab;
	Polynome res;

	tab = new Polynome[Q.N];

	for ( int i = 0 ; i < Q.N ; i++){

	    tab[i] = (Polynome) P.clone();
	    tab[i].mult(Q.a[i],Q.p[i]);

	}
	
	res=sum(tab);
	
	return res;
    }



    /**
     * division par une constante
     */
    public void  div( double k){
	for ( int i = 0 ; i <N  ;i++)
	    a[i]/=k;

    }


    
    /**
     * division par aX^n
     */
    public void div ( double coeff , double puiss){
	for ( int i = 0 ; i < N ; i++){
	    a[i]/=coeff;
	    p[i]-=puiss;
	}
    }



    /**
     * ne fonctionne pas
     */
    public static void div( double coeff,double puiss ,Polynome Q){
	for ( int i = 0 ; i < Q.N ; i++){
	    Q.a[i]=coeff/Q.a[i];
	    Q.p[i]=puiss-Q.a[i];
	}

    }



    /**
     * ne fonctionne pas
     */
    public static Polynome div( Polynome P,Polynome Q){
	Polynome [] tab;
	Polynome res;
	
	tab = new Polynome[P.N];
	
	for ( int i = 0 ; i < P.N ; i++){
	    
	    tab[i] = (Polynome) Q.clone();
	    div(P.a[i],P.p[i],tab[i]);
	    
	    
	}
	
	res=sum(tab);
	
	return res;
    }
    


    /**
     * retourne une copie du polynome
     */
    public Object clone(){
	
	double []coeff;
	double []puiss;
	Polynome res;

	coeff = new double [N];
	puiss = new double [N];
	
	System.arraycopy(a,0,coeff,0,N);
	System.arraycopy(p,0,puiss,0,N);

	res = new Polynome(coeff,puiss);

	return (Object)res;
    }
    


    /**
     * symplifie le polynome
     */
    public void factorise(){
	
	final double ZERO=0.0;
	
	for (int i = 0 ; i < N ; i++){
	    double puiss = p[i];
	    
	    
	    for ( int j = 0 ; j  < N ; j ++){

		if( i==j || a[j]==ZERO ||puiss!=p[j])
		    continue;

		/* si meme puiss => ajoute les coefficient */
		a[i] += a[j];
		
		/* met a jour le polynome */

		a[j] = ZERO;/*a[N-1];*/
		/*p[j] = 0.0;p[N-1];*/

		/*N--;j--;*/
	    }

	}
	
	/* on enleve tous les zeros */
	for ( int i =0 ; i< N ;i++)
	    if ( a[i]==ZERO) 
		{ a[i]=a[N-1];p[i]=p[N-1];N--;i--;} 
	
    }



    public String toString(){
	
	String str  = "[ ";


	if ( N>0)
	    str+=""+a[0]+"x^"+p[0];

	for ( int i= 1 ; i< N ; i++)

	    {  
		if ( a[i]>=0 ) 
		    str+=" + ";
		else
		    str+=" - ";
		
		if ( a[i]==0) 
		    str +="x^"+p[i];
		else 
		    str +=""+Math.abs(a[i])+"x^"+p[i];
	    }
	
	return str+" ]";
    }
    
    
    
    /**
     * retourne un polynome
     */
    public static Polynome getPolynome ( String str ){
	double [] coeff;
	double [] puiss;
	return null;
	
    }
   


    /**
     * calcul P(x)
     */
    public double P( double x){
	double res =0.0;
	for ( int i = 0 ; i< N ; i++)
	    res+= ( a[i] * Math.pow(x,p[i]) );
	
	return res;
    }



    
    /**
     * Calcul les P(x) pour une liste de nombre 
     */
    public double [] P(double [] X){
	double res[] =new double [X.length];
	
	for ( int i = 0 ; i < X.length ;i++)
	    res[i]=P(X[i]);
	
	return res;
    }



    public MyPolynome getMyPolynome(){
	double [] _coeff = new double[N];
	double [] _puiss =new double [N];

	System.arraycopy(a,0,_coeff,0,N);
	System.arraycopy(p,0,_puiss,0,N);

	return new MyPolynome(_coeff,_puiss);
    }

}/* fin de la class  */


